/**
 * @file
 * Client-side script to integrate private messages with Nodejs.
 */

/*global jQuery, Drupal, drupalSettings, io*/
/*jslint white:true, this, browser:true*/

(function ($, Drupal, drupalSettings, io) {

  "use strict";

  var initialized, threadSocket, inboxSocket, notificationSocket;

  function init() {

    // Only initialize once.
    if (!initialized) {
      initialized = true;

      /********** Begin thread handling **********/

      threadSocket = io(drupalSettings.privateMessageNodejs.nodejsUrl + "/pm_thread");

      threadSocket.on('connect', function () {
         // Enter for the room for this thread.
         threadSocket.emit('thread', drupalSettings.privateMessageThread.threadId);
      });

      // Listen for an emission informing of a new private message in the
      // thread.
      threadSocket.on("new private message", function () {
        // Fetch new messages from the server.
        Drupal.PrivateMessages.getNewMessages();
      });

      // Set up a callback for when the user has changed threads.
      Drupal.PrivateMessages.threadChange.privateMessageNodejs = {
        threadLoaded: function (threadId) {
          // Join the room for the new thread.
          threadSocket.emit('thread', threadId);
        }
      };

      /********** Begin inbox handling **********/

      if (Drupal.PrivateMessageInbox) {

        inboxSocket = io(drupalSettings.privateMessageNodejs.nodejsUrl + "/pm_inbox");

        // Listen for a connection to the remote server.
        inboxSocket.on('connect', function () {
           // Enter the room for this user.
           inboxSocket.emit('user', drupalSettings.user.uid);
        });

        // Listen for an emission informing of thread update for a thread the user
        // belongs to.
        inboxSocket.on("update pm inbox", function () {
          // Fetch new messages from the server.
          Drupal.PrivateMessageInbox.updateInbox();
        });
      }

      /********** Begin notification handling **********/

      if (Drupal.PrivateMessageNotificationBlock) {

        notificationSocket = io(drupalSettings.privateMessageNodejs.nodejsUrl + "/pm_notifications");

        // Listen for a connection to the remote server.
        notificationSocket.on('connect', function () {
           // Enter the room for this user.
           notificationSocket.emit('user', drupalSettings.user.uid);
        });

        // Listen for an emission informing of thread update for a thread the user
        // belongs to.
        notificationSocket.on("update pm unread thread count", function () {
          // Fetch unread thread count from the server.
          Drupal.PrivateMessageNotificationBlock.getUnreadThreadCount();
        });
      }

      /********** Begin ajax command callbacks handling **********/

      Drupal.AjaxCommands.prototype.privateMessageNodejsTriggerNewMessages = function () {
        threadSocket.emit("new private message", drupalSettings.privateMessageThread.threadId);
      };

      Drupal.AjaxCommands.prototype.privateMessageNodejsTriggerInboxUpdateCommand = function (ajax, response) {

        // For jSlint compatibility.
        ajax = ajax;

        if (Drupal.PrivateMessageInbox) {
          $.each(response.uids, function (index) {
            inboxSocket.emit("update pm inbox", response.uids[index]);
          });
        }
      };

      Drupal.AjaxCommands.prototype.privateMessageNodejsTriggerUnreadThreadCountUpdateCommand = function (ajax, response) {

        // For jSlint compatibility.
        ajax = ajax;

        if (Drupal.PrivateMessageNotificationBlock) {
          $.each(response.uids, function (index) {
            notificationSocket.emit("update pm unread thread count", response.uids[index]);
          });
        }
      };
    }
  }

  Drupal.behaviors.privateMessageNodejs = {
    attach: function () {
      init();
    }
  };

}(jQuery, Drupal, drupalSettings, io));
